package worldcontrolteam.worldcontrol.blocks;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.renderer.block.model.ModelResourceLocation;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.client.event.ModelRegistryEvent;
import net.minecraftforge.client.model.ModelLoader;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import worldcontrolteam.worldcontrol.client.ClientUtil;
import worldcontrolteam.worldcontrol.tileentity.TileEntityHowlerAlarm;
import worldcontrolteam.worldcontrol.utils.GuiLib;


public class BlockIndustrialAlarm extends BlockBasicRotate {

    private static final AxisAlignedBB UP_BLOCK_AABB = new AxisAlignedBB(0.125F, 0, 0.125F, 0.875F, 0.4375F, 0.875F);
    private static final AxisAlignedBB DOWN_BLOCK_AABB = new AxisAlignedBB(0.125F, 1, 0.125F, 0.875F, 0.5625F, 0.875F);
    private static final AxisAlignedBB EAST_BLOCK_AABB = new AxisAlignedBB(0F, 0.125F, 0.125F, 0.4375F, 0.875F, 0.875F);
    private static final AxisAlignedBB WEST_BLOCK_AABB = new AxisAlignedBB(1F, 0.125F, 0.125F, 0.5625F, 0.875F, 0.875F);
    private static final AxisAlignedBB NORTH_BLOCK_AABB = new AxisAlignedBB(0.125F, 0.125F, 0.5625F, 0.875F, 0.875F, 1);
    private static final AxisAlignedBB SOUTH_BLOCK_AABB = new AxisAlignedBB(0.125F, 0.125F, 0.4375F, 0.875F, 0.875F, 0);

    public BlockIndustrialAlarm() {
        this("industrial_alarm");
    }

    public BlockIndustrialAlarm(String name) {
        super(Material.field_151573_f, name);
        this.func_180632_j(this.func_176223_P().func_177226_a(FACING, EnumFacing.UP));
        this.defaultCreativeTab();
        func_149711_c(0.5F);

    }

    @Override
    @SideOnly(Side.CLIENT)
    public void registerModels(ModelRegistryEvent event) {
        final String resourcePath = getRegistryName().toString();
        ClientUtil.setCustomStateMapper(this, state -> new ModelResourceLocation(resourcePath, ClientUtil.getPropertyString(state.func_177228_b())));
        NonNullList<ItemStack> subBlocks = NonNullList.func_191196_a();
        func_149666_a(null, subBlocks);
        for (ItemStack stack : subBlocks) {
            IBlockState state = func_176223_P();
            ModelLoader.setCustomModelResourceLocation(Item.func_150898_a(this), stack.func_77960_j(), new ModelResourceLocation(resourcePath, ClientUtil.getPropertyString(state.func_177228_b())));
        }
    }

    @Override
    public TileEntity createTile(World world, int meta) {
        return new TileEntityHowlerAlarm();
    }

    @Override
    public boolean hasGUI() {
        return true;
    }

    @Override
    public int guiID() {
        return GuiLib.INDUSTRIAL_ALARM;
    }

    @Override
    public boolean func_149662_c(IBlockState state) {
        return false;
    }

    @Override
    public boolean func_149686_d(IBlockState state) {
        return false;
    }

    @Override
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos) {
        EnumFacing dir = state.func_177229_b(BlockBasicRotate.FACING);
        switch (dir) {
            case NORTH:
                return NORTH_BLOCK_AABB;
            case SOUTH:
                return SOUTH_BLOCK_AABB;
            case EAST:
                return EAST_BLOCK_AABB;
            case WEST:
                return WEST_BLOCK_AABB;
            case DOWN:
                return DOWN_BLOCK_AABB;
            default:
                return UP_BLOCK_AABB;
        }
    }

    @Override
    public void onNeighborChange(IBlockAccess world, BlockPos pos, BlockPos neighbor) {
        if (!(world.isSideSolid(pos.func_177972_a(world.func_180495_p(pos).func_177229_b(FACING).func_176734_d()), world.func_180495_p(pos).func_177229_b(FACING).func_176734_d(), true))) {
            this.func_176226_b((World) world, pos, world.func_180495_p(pos), 0);
            ((World) world).func_175698_g(pos);
        }
    }

    @Override
    public void func_189540_a(IBlockState state, World world, BlockPos pos, Block blockIn, BlockPos fromPos) {
        if (!(world.isSideSolid(pos.func_177972_a(world.func_180495_p(pos).func_177229_b(FACING).func_176734_d()), world.func_180495_p(pos).func_177229_b(FACING).func_176734_d()))) {
            this.func_176226_b(world, pos, world.func_180495_p(pos), 0);
            world.func_175698_g(pos);
        }
    }

    @Override
    public IBlockState getStateForPlacement(World world, BlockPos pos, EnumFacing facing, float hitX, float hitY, float hitZ, int meta, EntityLivingBase placer, EnumHand hand) {
        if (world.isSideSolid(pos.func_177972_a(facing.func_176734_d()), facing.func_176734_d())) {
            return this.func_176223_P().func_177226_a(FACING, facing);
        } else {
            if (world.func_180495_p(pos.func_177977_b()).func_177230_c() != Blocks.field_150350_a) {
                return this.func_176223_P().func_177226_a(FACING, EnumFacing.UP);
            } else {
                return null;
            }
        }
    }

    @Override
    public void func_180633_a(World world, BlockPos pos, IBlockState state, EntityLivingBase entity, ItemStack stack) {
        super.func_180633_a(world, pos, state, entity, stack);
        world.func_180501_a(pos, state, 2);
    }
}
