package worldcontrolteam.worldcontrol.blocks;

import com.google.common.collect.ImmutableMap;
import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.client.event.ModelRegistryEvent;
import net.minecraftforge.common.property.IExtendedBlockState;
import net.minecraftforge.common.property.IUnlistedProperty;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.tuple.ImmutablePair;
import worldcontrolteam.worldcontrol.client.ClientUtil;
import worldcontrolteam.worldcontrol.screen.IScreenContainer;
import worldcontrolteam.worldcontrol.tileentity.TileEntityInfoPanel;
import worldcontrolteam.worldcontrol.utils.WCUtility;

@SuppressWarnings("Duplicates")
public class BlockInfoPanel extends BlockBasicRotate implements IScreenContainer {
    private static ImmutableMap<EnumFacing, ImmutablePair<EnumFacing, EnumFacing>> facings;

    static {
        ImmutableMap.Builder<EnumFacing, ImmutablePair<EnumFacing, EnumFacing>> builder = new ImmutableMap.Builder<>();
        // dir, down, left
        builder.put(EnumFacing.UP, new ImmutablePair<>(EnumFacing.SOUTH, EnumFacing.WEST));
        builder.put(EnumFacing.DOWN, new ImmutablePair<>(EnumFacing.NORTH, EnumFacing.WEST));
        builder.put(EnumFacing.NORTH, new ImmutablePair<>(EnumFacing.DOWN, EnumFacing.EAST));
        builder.put(EnumFacing.SOUTH, new ImmutablePair<>(EnumFacing.DOWN, EnumFacing.WEST));
        builder.put(EnumFacing.EAST, new ImmutablePair<>(EnumFacing.DOWN, EnumFacing.SOUTH));
        builder.put(EnumFacing.WEST, new ImmutablePair<>(EnumFacing.DOWN, EnumFacing.NORTH));
        facings = builder.build();
    }

    public static class InfoPanelState {
        public boolean up, down, left, right, power;
        public int color;

        @Override
        public String toString() {
            return "asdf";
        }

        @Override
        public int hashCode() {
            return (up ? 1 : 0) + (down ? 2 : 0) + (left ? 4 : 0) + (right ? 8 : 0) + (power ? 16 : 0) + color << 5;
        }

        @Override
        public boolean equals(Object obj) {
            if (obj instanceof InfoPanelState) {
                InfoPanelState that = (InfoPanelState) obj;
                return that.power == this.power && that.up == this.up && this.down == that.down && this.left == that.left && this.right == that.right && this.color == that.color;
            }
            return false;
        }
    }
    public static final IUnlistedProperty<InfoPanelState> STATE = new IUnlistedProperty<InfoPanelState>() {
        @Override
        public String getName() {
            return "state";
        }

        @Override
        public boolean isValid(InfoPanelState infoPanelState) {
            return true;
        }

        @Override
        public Class<InfoPanelState> getType() {
            return InfoPanelState.class;
        }

        @Override
        public String valueToString(InfoPanelState infoPanelState) {
            return "asdf";
        }
    };

    public BlockInfoPanel(Material material, String name, boolean advanced) {
        super(material, name);
        defaultCreativeTab();
    }

    @Override
    public void func_180633_a(World world, BlockPos pos, IBlockState state, EntityLivingBase entity, ItemStack stack) {
        super.func_180633_a(world, pos, state, entity, stack);
        WCUtility.getTileEntity(world,pos, TileEntityInfoPanel.class).ifPresent(TileEntityInfoPanel::init);
    }

    public boolean isConnectedTo(IBlockAccess world, BlockPos p, BlockPos me) {
        Block block = world.func_180495_p(p).func_177230_c();
        return block instanceof IScreenContainer && WCUtility.compareBPos(((IScreenContainer) block).getOrigin(world, p),(me));
    }

    @Override
    public IBlockState getExtendedState(IBlockState state, IBlockAccess world, BlockPos pos) {
        if (state instanceof IExtendedBlockState) {
            IExtendedBlockState estate = (IExtendedBlockState) state;

            InfoPanelState istate = new InfoPanelState();
            istate.color = 3;
            istate.power = false;  // todo: get these values

            BlockPos origin = pos;
            if (origin != null) {
                TileEntity te_origin = world.func_175625_s(origin);
                if (te_origin instanceof TileEntityInfoPanel) {
                    istate.color = ((TileEntityInfoPanel) te_origin).color;
                    istate.power = ((TileEntityInfoPanel) te_origin).power;
                }
            }

            EnumFacing f = getFacing(world, pos);

            istate.down = isConnectedTo(world, pos.func_177972_a(facings.get(f).getKey()), pos);
            istate.up = isConnectedTo(world, pos.func_177972_a(facings.get(f).getKey().func_176734_d()), pos);
            istate.left = isConnectedTo(world, pos.func_177972_a(facings.get(f).getValue()), pos);
            istate.right = isConnectedTo(world, pos.func_177972_a(facings.get(f).getValue().func_176734_d()), pos);

            estate = estate.withProperty(STATE, istate);
            return estate;
        }
        else {
            return state;
        }
    }

    @Override
    public void registerModels(ModelRegistryEvent event) {
        ClientUtil.registerToNormalWithoutMapper(this);
    }

    @Override
    protected BlockStateContainer func_180661_e() {
        return new BlockStateContainer.Builder(this).add(FACING).add(STATE).build();
    }

    @Override
    public void func_180663_b(World world, BlockPos pos, IBlockState state) {
        WCUtility.getTileEntity(world, pos, TileEntityInfoPanel.class).ifPresent(te -> {
            te.updateAllProviders(true);
        });
        super.func_180663_b(world, pos, state);
    }

    @Override
    public TileEntity createTile(World world, int meta) {
        TileEntityInfoPanel tileEntityInfoPanel = new TileEntityInfoPanel();
        tileEntityInfoPanel.func_145834_a(world);
        return tileEntityInfoPanel;
    }

    @Override
    public boolean hasGUI() {
        return false;
    }

    @Override
    public int guiID() {
        return 0;
    }

    @Override
    public EnumFacing getFacing(IBlockAccess worldIn, BlockPos pos) {
        return worldIn.func_180495_p(pos).func_177230_c() == this ? (EnumFacing) worldIn.func_180495_p(pos).func_177228_b().get(FACING) : EnumFacing.DOWN;
    }

    @Override
    public BlockPos getOrigin(IBlockAccess worldIn, BlockPos pos) {
        return pos;
    }

    @Override
    public boolean isValid(World worldIn, BlockPos pos) {
        return true;
    }
}
