package worldcontrolteam.worldcontrol.client;

import net.minecraft.block.Block;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.renderer.block.model.ModelResourceLocation;
import net.minecraft.client.renderer.block.statemap.DefaultStateMapper;
import net.minecraft.client.renderer.block.statemap.IStateMapper;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.NonNullList;
import net.minecraftforge.client.model.ModelLoader;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import java.util.Iterator;
import java.util.Map;

@SideOnly(Side.CLIENT)
public class ClientUtil {

    public static void registerWithMapper(Block block) {
        if (block != null && block.getRegistryName() != null) {
            final String resourcePath = block.getRegistryName().toString();
            setCustomStateMapper(block, state -> new ModelResourceLocation(resourcePath, getPropertyString(state.func_177228_b())));
            NonNullList<ItemStack> subBlocks = NonNullList.func_191196_a();
            block.func_149666_a(null, subBlocks);
            for (ItemStack stack : subBlocks) {
                IBlockState state = block.func_176203_a(stack.func_77960_j());
                ModelLoader.setCustomModelResourceLocation(Item.func_150898_a(block), stack.func_77960_j(), new ModelResourceLocation(resourcePath, getPropertyString(state.func_177228_b())));
            }
        }
    }

    public static void setCustomStateMapper(Block block, StateMapper mapper) {
        ModelLoader.setCustomStateMapper(block, mapper.getMapper());
    }

    @SuppressWarnings("unchecked")
    public static <T extends Comparable<T>> String getPropertyName(IProperty<T> property, Comparable<?> comparable) {
        return property.func_177702_a((T) comparable);
    }

    public static String getPropertyString(Map<IProperty<?>, Comparable<?>> values, String... extrasArgs) {
        final StringBuilder stringbuilder = new StringBuilder();
        for (final Map.Entry<IProperty<?>, Comparable<?>> entry : values.entrySet()) {
            if (stringbuilder.length() != 0)
                stringbuilder.append(",");
            final IProperty<?> iProperty = entry.getKey();
            stringbuilder.append(iProperty.func_177701_a());
            stringbuilder.append("=");
            stringbuilder.append(getPropertyName(iProperty, entry.getValue()));
        }
        if (stringbuilder.length() == 0)
            stringbuilder.append("inventory");
        for (final String args : extrasArgs) {
            if (stringbuilder.length() != 0)
                stringbuilder.append(",");
            stringbuilder.append(args);
        }
        return stringbuilder.toString();
    }

    public static void registerToNormalWithoutMapper(Block block) {
        final String resourcePath = block.getRegistryName().toString();
        NonNullList<ItemStack> subBlocks = NonNullList.func_191196_a();
        block.func_149666_a(null, subBlocks);

        for (ItemStack stack : subBlocks) {
            ModelLoader.setCustomModelResourceLocation(Item.func_150898_a(block), stack.func_77960_j(), new ModelResourceLocation(resourcePath, "inventory"));
        }
    }

    public static void registerToNormal(Block block) {
        final String resourcePath = block.getRegistryName().toString();
        ModelLoader.setCustomStateMapper(block, new DefaultStateMapper() {
            protected ModelResourceLocation func_178132_a(IBlockState state) {
                return new ModelResourceLocation(resourcePath, "normal");
            }
        });
        NonNullList<ItemStack> subBlocks = NonNullList.func_191196_a();
        block.func_149666_a(null, subBlocks);
        Iterator var3 = subBlocks.iterator();

        while (var3.hasNext()) {
            ItemStack stack = (ItemStack) var3.next();
            ModelLoader.setCustomModelResourceLocation(Item.func_150898_a(block), stack.func_77960_j(), new ModelResourceLocation(resourcePath, "inventory"));
        }
    }

    public interface StateMapper {
        ModelResourceLocation getModelResourceLocation(IBlockState state);

        default IStateMapper getMapper() {
            return new DefaultStateMapper() {
                @Override
                protected ModelResourceLocation func_178132_a(IBlockState state) {
                    return StateMapper.this.getModelResourceLocation(state);
                }
            };
        }
    }

}
