package worldcontrolteam.worldcontrol.client.gui.features;


import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.FontRenderer;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.gui.ScaledResolution;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.client.FMLClientHandler;
import org.lwjgl.opengl.GL11;
import worldcontrolteam.worldcontrol.network.ChannelHandler;
import worldcontrolteam.worldcontrol.network.messages.PacketUpdateHowlerAlarm;
import worldcontrolteam.worldcontrol.tileentity.TileEntityHowlerAlarm;

import java.util.List;

public class HowlerAlarmListBox extends GuiButton {
    private static final String TEXTURE_FILE = "worldcontrol:textures/gui/gui_howler_alarm.png";
    private static final ResourceLocation TEXTURE_LOCATION = new ResourceLocation(TEXTURE_FILE);

    private static final int BASIC_X_OFFSET = 2;
    private static final int BASIC_Y_OFFSET = 2;
    private static final int SCROLL_WIDTH = 10;
    private static final int SCROLL_BUTTON_HEIGHT = 8;

    public int fontColor;
    public int selectedColor;
    public int selectedFontColor;
    public int lineHeight;
    public boolean dragging;
    private int scrollTop;
    private List<String> items;
    private TileEntityHowlerAlarm alarm;
    private int sliderHeight;
    private int sliderY;
    private int dragDelta;

    public HowlerAlarmListBox(int id, int left, int top, int width, int height, List<String> items, TileEntityHowlerAlarm alarm) {
        super(id, left, top, width, height, "");
        this.items = items;
        this.alarm = alarm;
        fontColor = 0x404040;
        selectedColor = 0xff404040;
        selectedFontColor = 0xA0A0A0;
        scrollTop = 0;
        lineHeight = 0;
        sliderHeight = 0;
        dragging = false;
        dragDelta = 0;
    }

    private static void draw(Tessellator tess, double x, double y, double z, float U, float V) {
        tess.func_178180_c().func_181662_b(x, y, z);
        tess.func_178180_c().func_187315_a(U, V);
        tess.func_178180_c().func_181666_a(1.0f, 1.0f, 1.0f, 1.0f);
        tess.func_178180_c().func_181675_d();
    }

    private void scrollTo(int pos) {
        scrollTop = pos;
        if (scrollTop < 0)
            scrollTop = 0;

        int max = lineHeight * items.size() + BASIC_Y_OFFSET - field_146121_g;

        if (max < 0)
            max = 0;

        if (scrollTop > max)
            scrollTop = max;
    }

    public void scrollUp() {
        scrollTop -= 8;

        if (scrollTop < 0)
            scrollTop = 0;
    }

    public void scrollDown() {
        scrollTop += 8;
        int max = lineHeight * items.size() + BASIC_Y_OFFSET - field_146121_g;

        if (max < 0)
            max = 0;

        if (scrollTop > max)
            scrollTop = max;
    }

    @Override
    public void func_191745_a(Minecraft minecraft, int cursorX, int cursorY, float i) {
        if (dragging) {
            int pos = (cursorY - field_146129_i - SCROLL_BUTTON_HEIGHT - dragDelta) * (lineHeight * items.size() + BASIC_Y_OFFSET - field_146121_g) /
                    Math.max(field_146121_g - 2 * SCROLL_BUTTON_HEIGHT - sliderHeight, 1);

            scrollTo(pos);
        }

        FontRenderer fontRenderer = minecraft.field_71466_p;
        String currentItem = alarm.getSound();
        if (lineHeight == 0) {
            lineHeight = fontRenderer.field_78288_b + 2;
            if (scrollTop == 0) {
                int rowsPerHeight = field_146121_g / lineHeight;
                int currentIndex = items.indexOf(currentItem);
                if (currentIndex >= rowsPerHeight)
                    scrollTop = (currentIndex + 1) * lineHeight + BASIC_Y_OFFSET - field_146121_g;
            }
            float scale = field_146121_g / ((float) lineHeight * items.size() + BASIC_Y_OFFSET);

            if (scale > 1)
                scale = 1;

            sliderHeight = Math.round(scale * (field_146121_g - 2 * SCROLL_BUTTON_HEIGHT));

            if (sliderHeight < 4)
                sliderHeight = 4;
        }

        int rowTop = BASIC_Y_OFFSET;
        GL11.glPushMatrix();
        GL11.glEnable(GL11.GL_SCISSOR_TEST);
        Minecraft mc = FMLClientHandler.instance().getClient();
        ScaledResolution scaler = new ScaledResolution(mc);
        GL11.glScissor(field_146128_h * scaler.func_78325_e(), mc.field_71440_d - (field_146129_i + field_146121_g) * scaler.func_78325_e(), (field_146120_f - SCROLL_WIDTH) * scaler.func_78325_e(), field_146121_g * scaler.func_78325_e());

        for (String row : items) {
            if (row.equals(currentItem)) {
                func_73734_a(field_146128_h, field_146129_i + rowTop - scrollTop - 1, field_146128_h + field_146120_f - SCROLL_WIDTH, field_146129_i + rowTop - scrollTop + lineHeight - 1, selectedColor);
                fontRenderer.func_78276_b(row, field_146128_h + BASIC_X_OFFSET, field_146129_i + rowTop - scrollTop, selectedFontColor);
            } else
                fontRenderer.func_78276_b(row, field_146128_h + BASIC_X_OFFSET, field_146129_i + rowTop - scrollTop, fontColor);

            rowTop += lineHeight;
        }

        GL11.glDisable(GL11.GL_SCISSOR_TEST);

        // Slider
        int sliderX = field_146128_h + field_146120_f - SCROLL_WIDTH + 1;
        sliderY = field_146129_i
                + SCROLL_BUTTON_HEIGHT
                + ((field_146121_g - 2 * SCROLL_BUTTON_HEIGHT - sliderHeight) * scrollTop)
                / (lineHeight * items.size() + BASIC_Y_OFFSET - field_146121_g);

        minecraft.field_71446_o.func_110577_a(TEXTURE_LOCATION);

        GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);

        func_73729_b(sliderX, sliderY, 131, 16, SCROLL_WIDTH - 1, 1);

        Tessellator tessellator = Tessellator.func_178181_a();
        tessellator.func_178180_c().func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_181709_i);
        this.draw(tessellator, (sliderX), sliderY + sliderHeight - 1, field_73735_i, 131 / 256F, (18) / 256F);
        this.draw(tessellator, sliderX + SCROLL_WIDTH - 1, sliderY + sliderHeight - 1, field_73735_i, (131 + SCROLL_WIDTH - 1) / 256F, (18) / 256F);
        this.draw(tessellator, sliderX + SCROLL_WIDTH - 1, sliderY + 1, field_73735_i, (131 + SCROLL_WIDTH - 1) / 256F, (17) / 256F);
        this.draw(tessellator, (sliderX), sliderY + 1, field_73735_i, 131 / 256F, (17) / 256F);
        tessellator.func_78381_a();

        func_73729_b(sliderX, sliderY + sliderHeight - 1, 131, 19, SCROLL_WIDTH - 1, 1);
        GL11.glPopMatrix();
    }

    private void setCurrent(int targetY) {
        if (lineHeight == 0)
            return;

        int itemIndex = (targetY - BASIC_Y_OFFSET - field_146129_i + scrollTop) / lineHeight;
        if (itemIndex >= items.size())
            itemIndex = items.size() - 1;

        String newSound = items.get(itemIndex);
        if (!newSound.equals(alarm.getSound())) {
            if (alarm.func_145831_w().field_72995_K)
                ChannelHandler.network.sendToServer(new PacketUpdateHowlerAlarm(alarm.getRange(), newSound, alarm.func_174877_v()));
            alarm.setSound(newSound);
        }
    }

    @Override
    public boolean func_146116_c(Minecraft minecraft, int targetX, int targetY) {
        if (super.func_146116_c(minecraft, targetX, targetY)) {
            if (targetX > field_146128_h + field_146120_f - SCROLL_WIDTH) {// scroll click

                if (targetY - field_146129_i < SCROLL_BUTTON_HEIGHT)
                    scrollUp();
                else if (field_146121_g + field_146129_i - targetY < SCROLL_BUTTON_HEIGHT)
                    scrollDown();
                else if (targetY >= sliderY && targetY <= sliderY + sliderHeight) {
                    dragging = true;
                    dragDelta = targetY - sliderY;
                }
            } else {
                setCurrent(targetY);

                return true;
            }
        } else {
            return false;
        }
        return false;
    }

    @Override
    public void func_146118_a(int i, int j) {
        super.func_146118_a(i, j);
        dragging = false;
    }
}
