package worldcontrolteam.worldcontrol.crossmod.industrialcraft2.blocks;

import ic2.api.energy.prefab.BasicSink;
import ic2.api.item.IElectricItem;
import ic2.api.reactor.IReactor;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.BlockPos;
import net.minecraftforge.items.IItemHandler;
import net.minecraftforge.items.IItemHandlerModifiable;
import net.minecraftforge.items.ItemHandlerHelper;
import worldcontrolteam.worldcontrol.crossmod.industrialcraft2.IC2Module;
import worldcontrolteam.worldcontrol.crossmod.industrialcraft2.ReactorUtils;
import worldcontrolteam.worldcontrol.crossmod.industrialcraft2.items.IC2ReactorCard;
import worldcontrolteam.worldcontrol.inventory.ISlotItemFilter;
import worldcontrolteam.worldcontrol.items.ItemUpgrade;
import worldcontrolteam.worldcontrol.tileentity.TileEntityBaseReactorHeatMonitor;
import worldcontrolteam.worldcontrol.utils.NBTUtils;
import worldcontrolteam.worldcontrol.utils.WCConfig;

import javax.annotation.Nonnull;

/**
 * Created by dmf444 on 10/25/2017. Code originally written for World-Control.
 */
public class TileEntityIC2RemoteReactorMonitor extends TileEntityBaseReactorHeatMonitor implements IItemHandler, IItemHandlerModifiable, ISlotItemFilter {

    private NonNullList<ItemStack> itemStack = NonNullList.<ItemStack>func_191197_a(5, ItemStack.field_190927_a);
    private static final double MIN_RANGE = 20;
    //TODO: Cap upgrade stacksize to 16 blocks
    private static final double ADDITIONAL_RANGE = 2.75f;

    private static final int BASE_STORAGE = 600;
    private static final int STORAGE_PER_UPGRADE = 10000;

    private BasicSink energySink;

    public TileEntityIC2RemoteReactorMonitor(){
        energySink = new BasicSink(this, BASE_STORAGE, 1);
    }

    @Override
    public void onLoad() {
        energySink.onLoad(); // notify the energy sink
    }

    @Override
    public void func_145843_s() {
        energySink.invalidate(); // notify the energy sink
        super.func_145843_s(); // this is important for mc!
    }

    @Override
    public void onChunkUnload() {
        energySink.onChunkUnload(); // notify the energy sink
    }

    @Override
    public int getCurrentHeat() {
        BlockPos ref = this.getReferenceBlock();
        if(ref != null) {
            IReactor reactor = ReactorUtils.getReactorAt(field_145850_b, ref);
            if (reactor != null) {
                return reactor.getHeat();
            }
        }
        return -1;
    }

    @Override
    public boolean isOverHeated() {
        if(energySink.useEnergy(WCConfig.remoteMonitorPowerUseIC2)){
            return super.isOverHeated();
        }
        return false;
    }

    @Override
    public boolean isConnectionValid() {
        if(itemStack.get(0) != ItemStack.field_190927_a){
            if(itemStack.get(0).func_77973_b() instanceof IC2ReactorCard){
                ItemStack card = itemStack.get(0);
                if (card.func_77942_o()) {
                    BlockPos pos = NBTUtils.getBlockPos(card.func_77978_p());
                    double distance = this.func_174877_v().func_185332_f(pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p());
                    if(distance <= ((itemStack.get(1).func_190916_E() - 1) * ADDITIONAL_RANGE) + MIN_RANGE) {
                        this.setReferenceBlock(pos);
                        return true;
                    }
                }
            }
        }
        return false;
    }

    @Override
    public int getSlots() {
        return this.itemStack.size();
    }

    @Nonnull
    @Override
    public ItemStack getStackInSlot(int slot) {
        return this.itemStack.get(slot);
    }

    @Nonnull
    @Override
    public ItemStack insertItem(int slot, @Nonnull ItemStack stack, boolean simulate) {
        //Modified from {@link SidedInvWrapper#extractItem}
        if (stack.func_190926_b())
            return ItemStack.field_190927_a;

        if (slot == -1)
            return stack;

        ItemStack stackInSlot = this.getStackInSlot(slot);

        int m;
        if (!stackInSlot.func_190926_b())
        {
            if (stackInSlot.func_190916_E() >= Math.min(stackInSlot.func_77976_d(), getSlotLimit(slot)))
                return stack;

            if (!ItemHandlerHelper.canItemStacksStack(stack, stackInSlot))
                return stack;


            m = Math.min(stack.func_77976_d(), getSlotLimit(slot)) - stackInSlot.func_190916_E();

            if (stack.func_190916_E() <= m) {
                if (!simulate) {
                    ItemStack copy = stack.func_77946_l();
                    copy.func_190917_f(stackInSlot.func_190916_E());
                    this.itemStack.set(slot, copy);
                }

                return stack;
            } else {
                // copy the stack to not modify the original one
                stack = stack.func_77946_l();
                if (!simulate) {
                    ItemStack copy = stack.func_77979_a(m);
                    copy.func_190917_f(stackInSlot.func_190916_E());
                    this.itemStack.set(slot, copy);
                    return stack;
                } else {
                    stack.func_190918_g(m);
                    return stack;
                }
            }
        } else {

            m = Math.min(stack.func_77976_d(), getSlotLimit(slot));
            if (m < stack.func_190916_E()) {
                // copy the stack to not modify the original one
                stack = stack.func_77946_l();
                if (!simulate) {
                    this.itemStack.set(slot, stack.func_77979_a(m));
                    return stack;
                } else {
                    stack.func_190918_g(m);
                    return stack;
                }
            } else {
                if (!simulate)
                    this.itemStack.set(slot,  stack);
                return ItemStack.field_190927_a;
            }
        }
    }

    @Nonnull
    @Override
    public ItemStack extractItem(int slot, int amount, boolean simulate) {
        //Modified from {@link SidedInvWrapper#extractItem}
        if(amount == 0)
            return ItemStack.field_190927_a;

        ItemStack stack = getStackInSlot(slot);
        if(stack.func_190926_b()){
            return ItemStack.field_190927_a;
        }

        if (simulate)
        {
            if (stack.func_190916_E() < amount)
            {
                return stack.func_77946_l();
            }
            else
            {
                ItemStack copy = stack.func_77946_l();
                copy.func_190920_e(amount);
                return copy;
            }
        }
        else
        {
            int m = Math.min(stack.func_190916_E(), amount);
            ItemStack ret = this.decrStackSize(slot, m);
            this.func_70296_d();
            return ret;
        }
    }

    @Override
    public int getSlotLimit(int slot) {
        return 64;
    }

    public ItemStack decrStackSize(int slot, int amount) {
        ItemStack stack = getStackInSlot(slot);
        if (stack != ItemStack.field_190927_a)
            if (stack.func_190916_E() > amount) {
                stack = stack.func_77979_a(amount);
                func_70296_d();
            } else {
                this.itemStack.set(slot, ItemStack.field_190927_a);
            }
        return stack;
    }

    public void setStackInSlot(int slot, @Nonnull ItemStack stack){
        this.itemStack.set(slot, stack);
        if (stack != ItemStack.field_190927_a && stack.func_190916_E() > this.getSlotLimit(slot)) {
            stack.func_190920_e(this.getSlotLimit(slot));
        }
    }

    public double getEnergy(){
        return this.energySink.getEnergyStored();
    }

    public void setEnergy(int energyIn){
        this.energySink.setEnergyStored(energyIn);
    }

    public double getMaxStorage(){
        return this.energySink.getCapacity();
    }

    @Override
    public void func_145839_a(NBTTagCompound nbttagcompound) {
        super.func_145839_a(nbttagcompound);
        energySink.readFromNBT(nbttagcompound);
        for(int i=0; i < itemStack.size(); i++){
            NBTTagCompound tag = nbttagcompound.func_74775_l("stack"+i);
            ItemStack stack = new ItemStack(tag);
            this.itemStack.set(i, stack);
        }
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound nbttagcompound) {
        nbttagcompound = super.func_189515_b(nbttagcompound);
        energySink.writeToNBT(nbttagcompound);
        for(int i=0; i < itemStack.size(); i++){
            ItemStack stack = this.itemStack.get(i);
            NBTTagCompound tag = stack.func_77955_b(new NBTTagCompound());
            nbttagcompound.func_74782_a("stack"+i, tag);
        }
        return nbttagcompound;
    }

    @Override
    public boolean isItemValid(int slotIndex, ItemStack itemstack) {
        switch (slotIndex) {
            case 1:
                if (Item.func_150891_b(itemstack.func_77973_b()) == Item.func_150891_b(IC2Module.suBattery.func_77973_b()))
                    return true;
                if (itemstack.func_77973_b() instanceof IElectricItem) {
                    IElectricItem item = (IElectricItem) itemstack.func_77973_b();
                    if (item.canProvideEnergy(itemstack) && item.getTier(itemstack) <= this.energySink.getSinkTier()) {
                        return true;
                    }
                }
                return false;
            case 0:
                return itemstack.func_77973_b() instanceof IC2ReactorCard;// || itemstack.getItem() instanceof ItemCard55Reactor;
            default:
                return itemstack.func_77969_a(IC2Module.transformerUpgrade)
                        || itemstack.func_77969_a(IC2Module.energyUpgrade)
                        || (itemstack.func_77973_b() instanceof ItemUpgrade && itemstack.func_77952_i() == ItemUpgrade.DAMAGE_RANGE);
        }
    }
}
