package worldcontrolteam.worldcontrol.tileentity;

import com.google.common.collect.Lists;
import net.minecraft.client.Minecraft;
import net.minecraft.client.audio.PositionedSoundRecord;
import net.minecraft.entity.Entity;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.play.server.SPacketUpdateTileEntity;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.ITickable;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.SoundEvent;
import net.minecraft.util.math.BlockPos;
import net.minecraftforge.fml.client.FMLClientHandler;
import net.minecraftforge.fml.common.FMLCommonHandler;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import worldcontrolteam.worldcontrol.WorldControl;
import worldcontrolteam.worldcontrol.utils.RedstoneHelper;
import worldcontrolteam.worldcontrol.utils.WCConfig;
import worldcontrolteam.worldcontrol.utils.WCUtility;


public class TileEntityHowlerAlarm extends TileEntity implements ITickable, RedstoneHelper.IRedstoneConsumer {

    private static final String DEFAULT_SOUND_NAME = "default";
    private static final String SOUND_PREFIX = "worldcontrol:alarm-";
    private static final float BASE_SOUND_RANGE = 16F;
    protected int tickRate;
    private String soundName;
    private int color;
    private int range;
    private int updateTicker;
    private boolean init = false;
    private boolean powered;
    @SideOnly(Side.CLIENT)
    private TileEntitySound sound;


    public TileEntityHowlerAlarm() {
        tickRate = 2;
        updateTicker = 0;
        soundName = DEFAULT_SOUND_NAME;
        range = WCConfig.alarmRange;
        color = WCUtility.WHITE;
        if (WorldControl.SIDE == Side.CLIENT) {
            sound = new TileEntitySound(BASE_SOUND_RANGE);
        }
    }

    @Override
    public boolean getPowered() {
        return powered;
    }

    @Override
    public void setPowered(boolean value) {
        powered = value;

        if (powered) {
            if (FMLCommonHandler.instance().getEffectiveSide().isClient())
                sound.playAlarm(func_174877_v().func_177963_a(0.5, 0.5, 0.5), SOUND_PREFIX + soundName, getNormalizedRange(), false);
        } else {
            if (FMLCommonHandler.instance().getEffectiveSide().isClient()) {
                sound.stopAlarm();
            }
        }
    }

    @Override
    public void func_73660_a() {
        // WCUtility.error(color);
        if (!init) {
            if (!func_145831_w().field_72995_K)
                RedstoneHelper.checkPowered(func_145831_w(), this);
            init = true;
        }
        if (FMLCommonHandler.instance().getEffectiveSide().isClient()) {
            if (tickRate != -1 && updateTicker-- > 0)
                return;
            updateTicker = tickRate;
            checkStatus();
        }
    }

    protected void checkStatus() {
        RedstoneHelper.checkPowered(func_145831_w(), this);
        if (FMLCommonHandler.instance().getEffectiveSide().isClient()) {
            if (powered && !sound.isPlaying()) {
                sound.playAlarm(this.func_174877_v().func_177963_a(0.5, 0.5, 0.5), SOUND_PREFIX + soundName, getNormalizedRange(), true);
            }
        }
    }

    private float getNormalizedRange() {
        if (func_145831_w().field_72995_K) {
            return Math.min(range, WCConfig.SMPmaxAlarmRange) / BASE_SOUND_RANGE;
        }
        return range / BASE_SOUND_RANGE;
    }

    public NBTTagCompound func_189517_E_() {
        return this.func_189515_b(new NBTTagCompound());
    }

    @Override
    public SPacketUpdateTileEntity func_189518_D_() {
        NBTTagCompound syncData = new NBTTagCompound();
        syncData.func_74778_a("soundName", soundName);
        syncData.func_74768_a("range", range);
        syncData.func_74768_a("color", color);
        syncData.func_74757_a("powered", powered);
        return new SPacketUpdateTileEntity(this.func_174877_v(), 1, syncData);
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void onDataPacket(net.minecraft.network.NetworkManager net, SPacketUpdateTileEntity pkt) {
        color = pkt.func_148857_g().func_74762_e("color");
        range = pkt.func_148857_g().func_74762_e("range");
        soundName = pkt.func_148857_g().func_74779_i("soundName");
        powered = pkt.func_148857_g().func_74767_n("powered");
    }


    @Override
    public void func_145839_a(NBTTagCompound nbttagcompound) {
        super.func_145839_a(nbttagcompound);
        color = nbttagcompound.func_74762_e("color");
        if (nbttagcompound.func_74764_b("soundName")) {
            soundName = nbttagcompound.func_74779_i("soundName");
            range = nbttagcompound.func_74762_e("range");
        }
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound nbttagcompound) {
        nbttagcompound = super.func_189515_b(nbttagcompound);
        nbttagcompound.func_74778_a("soundName", soundName);
        nbttagcompound.func_74768_a("range", range);
        nbttagcompound.func_74768_a("color", color);
        return nbttagcompound;
    }

    public int getRange() {
        return range;
    }

    public void setRange(int range) {
        this.range = range;
    }

    public String getSound() {
        return soundName;
    }

    public void setSound(String sound) {
        if (Lists.newArrayList(WCConfig.howlerAlarmSounds).contains(sound)) {
            this.soundName = sound;
        } else {
            this.soundName = DEFAULT_SOUND_NAME;
        }
    }

    public int getColor() {
        return color;
    }

    public void setColor(int color) {
        this.color = color;
    }


    @SideOnly(Side.CLIENT)
    public class TileEntitySound {
        private PositionedSoundRecord sound;
        private float DEFAULT_RANGE;

        public TileEntitySound(float ranger) {
            DEFAULT_RANGE = ranger;
        }

        public void stopAlarm() {
            if (sound != null) {
                Minecraft.func_71410_x().func_147118_V().func_147683_b(sound);
                sound = null;
            }
        }

        public void playAlarm(BlockPos pos, String soundName, float range, boolean skipCheck) {
            if (sound == null || skipCheck) {
                sound = playSound(pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p(), soundName, range);
            }
        }

        public PositionedSoundRecord playSound(double x, double y, double z, String name, float volume) {
            float range = DEFAULT_RANGE;

            if (volume > 1.0F) {
                range *= volume;
            }

            Entity person = FMLClientHandler.instance().getClient().func_175606_aa();

            if (person != null && volume > 0 && person.func_70092_e(x, y, z) < range * range) {
                PositionedSoundRecord sound = new PositionedSoundRecord(new SoundEvent(new ResourceLocation(name)), SoundCategory.BLOCKS, volume, 1.0F, (float) x, (float) y, (float) z);
                Minecraft.func_71410_x().func_147118_V().func_147682_a(sound);
                return sound;
            }
            return null;
        }

        public boolean isPlaying() {
            if (sound == null) {
                return false;
            }
            return Minecraft.func_71410_x().func_147118_V().func_147692_c(sound);
        }
    }
}
