package worldcontrolteam.worldcontrol.tileentity;

import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.NetworkManager;
import net.minecraft.network.play.server.SPacketUpdateTileEntity;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ITickable;
import net.minecraft.util.math.BlockPos;
import worldcontrolteam.worldcontrol.api.card.ICard;
import worldcontrolteam.worldcontrol.blocks.BlockInfoPanel;
import worldcontrolteam.worldcontrol.init.WCContent;
import worldcontrolteam.worldcontrol.screen.IScreenContainer;
import worldcontrolteam.worldcontrol.api.screen.IScreenElement;
import worldcontrolteam.worldcontrol.utils.WCUtility;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by dmf444 on 8/15/2017. Code originally written for World-Control.
 */
public class TileEntityInfoPanel extends TileEntity implements IInventory, ITickable {
    public int color;
    public boolean power;

    public BlockPos origin;
    public BlockPos end;

    public ItemStack card;
    public IScreenElement ise;

    public EnumFacing facing;

    private Map<BlockPos, Boolean> validCache = new HashMap<>();

    public TileEntityInfoPanel() {
        this.color = 10;
        this.power = true;

        // debug debug debug todo: fixme: aaaaaaa

        card = new ItemStack(WCContent.TIME_CARD, 1);
        func_174886_c(null);
    }

    public void init() {
        this.facing = (EnumFacing) field_145850_b.func_180495_p(func_174877_v()).func_177228_b().get(BlockInfoPanel.FACING);
        if (origin == null) origin = func_174877_v();
        if (end == null) end = func_174877_v();

        searchForNeighbours();
        updateAllProviders(false);
    }

    public void reInit() {
        updateAllProviders(true);
        searchForNeighbours();
        updateAllProviders(false);
    }

    private boolean isAreaValid(BlockPos o, BlockPos e) {
        for (int x = o.func_177958_n(); x <= e.func_177958_n(); x++) {
            for (int y = o.func_177956_o(); y <= e.func_177956_o(); y++) {
                for (int z = o.func_177952_p(); z <= e.func_177952_p(); z++) {
                    final BlockPos pos1 = new BlockPos(x, y, z);
                    if (validCache.containsKey(pos1)) {
                        if (!validCache.get(pos1)) return false;
                        continue;
                    }
                    if (!(field_145850_b.func_180495_p(pos1).func_177230_c() instanceof IScreenContainer)) {
                        validCache.put(pos1, false);
                        return false;
                    }
                    IScreenContainer c = (IScreenContainer) field_145850_b.func_180495_p(pos1).func_177230_c();
                    if (!(c.getFacing(field_145850_b, pos1) == facing)) {
                        validCache.put(pos1, false);
                        return false;
                    }
                    if (!c.isValid(field_145850_b, pos1)) {
                        validCache.put(pos1, false);
                        return false;
                    }
                    if (c.getOrigin(field_145850_b, pos1) != null && !WCUtility.compareBPos(c.getOrigin(field_145850_b, pos1), func_174877_v())) {
                        validCache.put(pos1, false);
                        return false;
                    }
                    validCache.put(pos1, true);
                }
            }
        }
        return true;
    }
    
    public void updateAllProviders(boolean nullify) {
        for (int x = origin.func_177958_n(); x <= end.func_177958_n(); x++) {
            for (int y = origin.func_177956_o(); y <= end.func_177956_o(); y++) {
                for (int z = origin.func_177952_p(); z <= end.func_177952_p(); z++) {
                    final BlockPos pos = new BlockPos(x, y, z);
                    if (field_145850_b.func_180495_p(pos).func_177230_c() instanceof IScreenContainer) {
                        if (nullify) {
                            ((IScreenContainer) (field_145850_b.func_180495_p(pos).func_177230_c())).setOrigin(field_145850_b, pos, null);
                        }
                        else {
                            ((IScreenContainer) (field_145850_b.func_180495_p(pos).func_177230_c())).setOrigin(field_145850_b, pos, func_174877_v());
                        }
                    }
                }
            }
        }
    }


    private boolean inSamePlane(BlockPos c) {
        switch (facing) {
            case UP:
            case DOWN:
                return c.func_177956_o() == origin.func_177956_o();
            case NORTH:
            case SOUTH:
                return c.func_177952_p() == origin.func_177952_p();
            default:
                return c.func_177958_n() == origin.func_177958_n();
        }
    }

    public void tryToAdd(BlockPos aPos) {
        if (!inSamePlane(aPos)) {
            return;
        }
        updateAllProviders(true);
        searchForNeighbours();
        updateAllProviders(false);
    }

    private void searchForNeighbours() {
        validCache = new HashMap<>();
        EnumFacing left = EnumFacing.WEST; // negRelativeX
        EnumFacing down = EnumFacing.DOWN; // negRealtiveY

        switch (facing) {
            case UP:
            case DOWN:
                left = EnumFacing.WEST;
                down = EnumFacing.NORTH;
                break;
            case EAST:
            case WEST:
                left = EnumFacing.NORTH;
                down = EnumFacing.DOWN;
                break;
            case NORTH:
            case SOUTH:
                left = EnumFacing.WEST;
                down = EnumFacing.DOWN;
                break;
        }
        if (!isAreaValid(origin, end)) {
            origin = func_174877_v();
            end = func_174877_v();
        }
        int n = 0;
        do {
            origin = origin.func_177972_a(left);
            n += 1;
        } while (isAreaValid(origin, end.func_177967_a(left, n)));


        n = 0;

        origin = origin.func_177972_a(left.func_176734_d());

        do {
            origin = origin.func_177972_a(down);
            n += 1;
        } while (isAreaValid(origin, end.func_177967_a(down, n)));


        origin = origin.func_177972_a(down.func_176734_d());
        n = 0;

        do {
            end = end.func_177972_a(left.func_176734_d());
            n += 1;
        } while (isAreaValid(origin.func_177967_a(left.func_176734_d(), n), end));


        end = end.func_177972_a(left);
        n = 0;

        do {
            end = end.func_177972_a(down.func_176734_d());
            n += 1;
        } while (isAreaValid(origin.func_177967_a(down.func_176734_d(), n), end));


        end = end.func_177972_a(down);
        IBlockState bs = field_145850_b.func_180495_p(func_174877_v());
        field_145850_b.func_184138_a(func_174877_v(), bs, bs, 0);
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound compound) {
        compound = super.func_189515_b(compound);

        if (origin == null) origin = func_174877_v();
        if (end == null) end = func_174877_v();
        if (facing == null) facing = EnumFacing.NORTH;

        compound.func_74768_a("oX", origin.func_177958_n());
        compound.func_74768_a("oY", origin.func_177956_o());
        compound.func_74768_a("oZ", origin.func_177952_p());

        compound.func_74768_a("eX", end.func_177958_n());
        compound.func_74768_a("eY", end.func_177956_o());
        compound.func_74768_a("eZ", end.func_177952_p());

        compound.func_74768_a("facing", facing.func_176745_a());
        compound.func_74757_a("power", power);
        compound.func_74768_a("color", color);

        return compound;
    }

    @Override
    public void func_145839_a(NBTTagCompound compound) {
        super.func_145839_a(compound);

        origin = new BlockPos(
                compound.func_74762_e("oX"),
                compound.func_74762_e("oY"),
                compound.func_74762_e("oZ"));

        end = new BlockPos(
                compound.func_74762_e("eX"),
                compound.func_74762_e("eY"),
                compound.func_74762_e("eZ"));

        facing = EnumFacing.func_82600_a(compound.func_74762_e("facing"));
        power = compound.func_74767_n("power");
        color = compound.func_74762_e("color");

        //updateAllProviders(false);
    }

    @Nullable
    @Override
    public SPacketUpdateTileEntity func_189518_D_() {
        NBTTagCompound compound = new NBTTagCompound();

        compound.func_74768_a("oX", origin.func_177958_n());
        compound.func_74768_a("oY", origin.func_177956_o());
        compound.func_74768_a("oZ", origin.func_177952_p());

        compound.func_74768_a("eX", end.func_177958_n());
        compound.func_74768_a("eY", end.func_177956_o());
        compound.func_74768_a("eZ", end.func_177952_p());

        compound.func_74768_a("facing", facing.func_176745_a());
        compound.func_74757_a("power", power);
        compound.func_74768_a("color", color);

        return new SPacketUpdateTileEntity(func_174877_v(), 3, compound);
    }

    @Override
    public NBTTagCompound func_189517_E_() {
        NBTTagCompound compound = super.func_189517_E_();

        compound.func_74768_a("oX", origin.func_177958_n());
        compound.func_74768_a("oY", origin.func_177956_o());
        compound.func_74768_a("oZ", origin.func_177952_p());

        compound.func_74768_a("eX", end.func_177958_n());
        compound.func_74768_a("eY", end.func_177956_o());
        compound.func_74768_a("eZ", end.func_177952_p());

        compound.func_74768_a("facing", facing.func_176745_a());
        compound.func_74757_a("power", power);
        compound.func_74768_a("color", color);

        return compound;
    }

    @Override
    public void onDataPacket(NetworkManager net, SPacketUpdateTileEntity pkt) {
        NBTTagCompound compound = pkt.func_148857_g();

        origin = new BlockPos(
                compound.func_74762_e("oX"),
                compound.func_74762_e("oY"),
                compound.func_74762_e("oZ"));

        end = new BlockPos(
                compound.func_74762_e("eX"),
                compound.func_74762_e("eY"),
                compound.func_74762_e("eZ"));

        facing = EnumFacing.func_82600_a(compound.func_74762_e("facing"));
        power = compound.func_74767_n("power");
        color = compound.func_74762_e("color");
    }

    @Override
    public int func_70302_i_() {
        return 1;
    }

    @Override
    public boolean func_191420_l() {
        return card == null;
    }

    @Override
    public ItemStack func_70301_a(int i) {
        return card;
    }

    @Override
    public ItemStack func_70298_a(int i, int i1) {
        if (card != null && i1 > 0) {
            this.card = null;
        }
        return this.card;
    }

    @Override
    public ItemStack func_70304_b(int i) {
        return null;
    }

    @Override
    public void func_70299_a(int i, ItemStack itemStack) {
        if (!func_94041_b(0, itemStack)) return;
        this.card = itemStack;
    }

    @Override
    public int func_70297_j_() {
        return 1;
    }

    @Override
    public boolean func_70300_a(EntityPlayer entityPlayer) {
        return true;
    }

    @Override
    public void func_174889_b(EntityPlayer entityPlayer) {

    }

    @Override
    public void func_174886_c(EntityPlayer entityPlayer) {
        if (this.card == null) {
            ise = null;
        }
        else {
            ise = ((ICard)this.card.func_77973_b()).getRenderer(this.card);
        }
    }

    @Override
    public boolean func_94041_b(int i, ItemStack itemStack) {
        return itemStack.func_77973_b() instanceof ICard;
    }

    @Override
    public int func_174887_a_(int i) {
        return 0;
    }

    @Override
    public void func_174885_b(int i, int i1) {

    }

    @Override
    public int func_174890_g() {
        return 0;
    }

    @Override
    public void func_174888_l() {
        card = null;
    }

    @Override
    public String func_70005_c_() {
        return "Info Panel";
    }

    @Override
    public boolean func_145818_k_() {
        return false;
    }

    @Override
    public void func_73660_a() {
        if (this.card != null) {
            ICard icard = (ICard) this.card.func_77973_b();
            icard.update(field_145850_b, this.card);
            ise.onCardUpdate(field_145850_b, this.card);
        }
    }
}
